/*
  gammadialog.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "gammadialog.h"
#include "mainwindow.h"
#include <qlayout.h>
#include <qslider.h>
#include <qlabel.h>

#define DIALOG_WIDTH  400
#define DIALOG_HEIGHT 10

#define SLIDER_STEP 5
#define STEP(val) ((val)/SLIDER_STEP)

GammaDialog::GammaDialog( QWidget* parent, const char* name,
						  bool modal, WFlags fl)
	: QDialog( parent, name, modal, fl )
{
    resize( QSize(DIALOG_WIDTH, DIALOG_HEIGHT).expandedTo(minimumSizeHint()) );

	setCaption(QString::fromUtf8("ガンマ補正"));
	setFont(QFont("lcfont", 16));

	base_hbox = new QHBoxLayout(this);
	slider = new QSlider(STEP(10), STEP(100), STEP(10), STEP(10),
						 QSlider::Horizontal, this);
	label = new QLabel(this);
	base_hbox->addWidget(slider);
	base_hbox->addWidget(label);

	connect(slider, SIGNAL(valueChanged(int)),
			this, SLOT(changeSlider(int)));
	connect(this, SIGNAL(reload(bool)),
			parent, SIGNAL(reload(bool)));
	connect(this, SIGNAL(cacheFlush()),
			parent, SIGNAL(cacheFlush()));
}

GammaDialog::~GammaDialog()
{
}

void GammaDialog::setDefault(double value)
{
	def_value = value;
	slider->setValue(STEP((int)(value*100)));
}

double GammaDialog::getGamma()
{
	return (double)(slider->value()*SLIDER_STEP) / 100.0;
}

void GammaDialog::changeSlider(int value)
{
	QString str;
	value *= SLIDER_STEP;
	str.sprintf("%d.%02d", value/100, value %100);
	label->setText(str);

	killTimers();
	startTimer(500);
}

void GammaDialog::reject(void)
{
	changeSlider(STEP(def_value*100));
	QDialog::reject();
}

void GammaDialog::timerEvent(QTimerEvent *e)
{
	killTimer(e->timerId());

	QGConf *conf = (QGConf *)lookupObject(this, "QGConf", "config");
	if(conf) {
		conf->writeEntry("Gamma", label->text());
		emit cacheFlush();
		emit reload(true);
	}
}
