/*
  layeredbase.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __LAYEREDBASE_H__
#define __LAYEREDBASE_H__

#include "layeredwidget.h"
#include <qwidget.h>
#include <qarray.h>
#include <qevent.h>
#include <qrect.h>
#include <qdatetime.h>
#include "qgmap.h"

class CustomRepaintEvent : public QCustomEvent
{
  public:
	CustomRepaintEvent(QRect r) : QCustomEvent(EV_CUSTOM_REPAINT), _rect(r) {};
	QRect rect() {return _rect;};
  private:
	QRect _rect;
};

class LayeredBase : public QWidget
{
  public:
	LayeredBase( QWidget *parent =0, const char *name = 0, WFlags fl = 0);

	void addWidget( LayeredWidget *widget );

	void repaint( void );
	void repaint( int x, int y, int w, int h );

	void sendRequest(QString const &req, QString const &arg);

  protected:
	void paintEvent( QPaintEvent *e );

	void keyPressEvent ( QKeyEvent * e );
	void keyReleaseEvent ( QKeyEvent * e );
	void wheelEvent( QWheelEvent *e );
	void mouseMoveEvent( QMouseEvent *e );
	void mousePressEvent( QMouseEvent *e );
	void mouseReleaseEvent( QMouseEvent *e );
	void mouseDoubleClickEvent( QMouseEvent *e );
	void showEvent( QShowEvent *e );
	void customEvent( QCustomEvent *e );
	void resizeEvent( QResizeEvent *e );

  private:
	QTime press_time;
	QPoint press_pos;
	QTime dbl_click_time;
	bool dbl_click;
	QPoint first_press_pos;

  protected:
	QArray<LayeredWidget *> array;
	int clicked_layer;

	void blockPaint();
	void unblockPaint();

	bool paint_request;

	bool initialized;
};

#endif // __LAYEREDBASE_H__
